/**
 * Title :        XLogo
 * Description :  XLogo is an interpreter for the Logo 
 * 						programming language
 * @author Loïc Le Coq
 */

import java.io.*;
import java.util.StringTokenizer;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


public class Lanceur {
	private File destination=null;
	private int memoire=64;
	Lanceur(String[] args){
		String arguments=" ";
		for (int i=0;i<args.length;i++){
			arguments+=args[i]+"\u001B";
		}
		arguments.trim();
		lis_config();
		extrait_application();
		try{
			String commande=System.getProperty("java.home")+File.separator+"bin"+File.separator+"java -jar -Xmx"+memoire+"m "+destination.getAbsolutePath()+arguments;
			System.out.println("Launch XLogo ---->"+commande);
			Process p=Runtime.getRuntime().exec(commande);
			p.waitFor();
	//		while(true) Thread.currentThread().sleep(1000);
		}
		catch(InterruptedException e3){System.out.println(e3);}
		catch(IOException e1){System.out.println(e1);}
		catch(SecurityException e2){System.out.println(e2);}
		destination.delete();
		System.out.println("Closing XLogo. Cleaning tmp file");
	}
	private void extrait_application(){
		try{
			
			destination=File.createTempFile("tmp_xlogo",".jar");
			copier(destination);

			//	destination.deleteOnExit();
		}
		catch(IOException e2){}
	}
	 void lis_config(){
	 	try{
		 // Try to read XML format (new config file)
			FileInputStream fr = new FileInputStream(System.getProperty("user.home")+File.separator+".xlogo");
			BufferedInputStream bis = new BufferedInputStream(fr);	
		    InputStreamReader isr = new  InputStreamReader(bis,  "UTF8");		
		    try{
		     	XMLReader saxReader = XMLReaderFactory.createXMLReader();
		      	saxReader.setContentHandler(new MemoryContentHandler());
		      	saxReader.parse(new InputSource(isr));
		    }
		     catch (SAXException e){
		 			// Read the old config file format
		    	 	String s="";
		    	 	FileReader ifr = new FileReader(System.getProperty("user.home")+File.separator+".xlogo");
		    	 	while(ifr.ready()){
		    	 		char[] b = new char[64];
		    	 		int i=ifr.read(b);
		    	 		if (i==-1) break;
		    	 		s+=new String(b);
		    	 	}
		    	 	StringTokenizer st=new StringTokenizer(s,"\n");
		    	 	while(st.hasMoreTokens()){
		    	 		String element=st.nextToken();
		    	 		if (element.equals("# memoire")){
		    	 			element=st.nextToken();
		    	 			memoire=Integer.parseInt(element);
		    	 		}
		    	 	}
		     	}
	 		}
	 		catch(FileNotFoundException e1){System.out.println("No File \".xlogo\". Will create one...");}
	 		catch(IOException e2){}
	 }
	 		
	 		
	public static void main(String[] args) {
	new Lanceur(args);
	}
	 /* retourne vrai si cela réussit
	 */
	public static boolean copier(File destination )
	{
	        boolean resultat = false;
	        // Declaration des flux
	        java.io.InputStream sourceFile=null;
	        java.io.FileOutputStream destinationFile=null;
	        try {
	                // Création du fichier :
	                destination.createNewFile();
	                // Ouverture des flux
	                sourceFile = Lanceur.class.getResourceAsStream("tmp_xlogo.jar");//new java.io.FileInputStream(source);
	                destinationFile = new java.io.FileOutputStream(destination);
	                // Lecture par segment de 0.5Mo 
	                byte buffer[]=new byte[512*1024];
	                int nbLecture;
	                while( (nbLecture = sourceFile.read(buffer)) != -1 ) {
	                        destinationFile.write(buffer, 0, nbLecture);
	                } 
	                // Copie réussie
	                resultat = true;
	        } catch( java.io.FileNotFoundException f ) {
	        } catch( java.io.IOException e ) {
	        } finally {
	                // Quoi qu'il arrive, on ferme les flux
	                try {
	                        sourceFile.close();
	                } catch(Exception e) { }
	                try {
	                        destinationFile.close();
	                } catch(Exception e) { }
	        } 
	        return( resultat );
	}
	class MemoryContentHandler implements ContentHandler{
		//   private Locator locator;
		    
	  		public MemoryContentHandler() {
	  			    super();
	                  // We define default Locator
	        //          locator = new LocatorImpl();
	          }
	          public void setDocumentLocator(Locator value) {
	          //        locator =  value;
	          }
	          public void startDocument() throws SAXException {
	          }
	          public void endDocument() throws SAXException {
	          }

	          public void startPrefixMapping(String prefix, String URI) throws SAXException {
	          }
	          public void endPrefixMapping(String prefix) throws SAXException {
	          }
	          // Open new Tag 
	          public void startElement(String nameSpaceURI, String localName, String rawName, Attributes attributs) throws SAXException {
	                  analyseBalise(localName,attributs);
	/*                   System.out.println("Ouverture de la balise: "+localName);
	                  for (int index = 0; index < attributs.getLength(); index++) { // on parcourt la liste des attributs
	                          System.out.println("     - " +  attributs.getLocalName(index) + " = " + attributs.getValue(index));
	                  }*/
	          }
	          
	          // Close tag
	          public void endElement(String nameSpaceURI, String localName, String rawName) throws SAXException {
	       /*           System.out.print("Fermeture de la balise : " + localName);

	                  if ( ! "".equals(nameSpaceURI)) { // name space non null
	                          System.out.print("appartenant a l'espace de nommage : " + localName);
	                  }

	                  System.out.println();*/
	          }
	          
	          public void characters(char[] ch, int start, int end) throws SAXException {
	          }

	          public void ignorableWhitespace(char[] ch, int start, int end) throws SAXException {
	          }
	          public void processingInstruction(String target, String data) throws SAXException {
	          }
	          public void skippedEntity(String arg0) throws SAXException {
	          }
	          private void analyseBalise(String tag,Attributes attributs){
	          	if (tag.equals("memory")){
	          		memoire=Integer.parseInt(attributs.getValue(0));
	          	}
	       }

	}
	
}
